// DESCRIPTION: Verilator: Verilog Test module
//
// This file ONLY is placed under the Creative Commons Public Domain.
// SPDX-FileCopyrightText: 2020 Wilson Snyder
// SPDX-License-Identifier: CC0-1.0

class Other;
endclass

class Cls;
  int imembera;
  function int inc_methoda;
    imembera += 1;
    return imembera;
  endfunction
endclass

module t;
  initial begin
    Cls c1;
    Other co;
    c1 = new co;  // Bad, incompatible types
  end
endmodule
