#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('simulator')

test.top_filename = "t/t_procedure_nba.v"

test.compile(verilator_flags2=["--binary", "--stats", "-DPROCEDURE=initial"])

test.file_grep(test.stats, r'Scheduling, \'act\' extra triggers\s+(\d+)', 0)
test.file_grep(test.stats, r'Scheduling, \'act\' pre triggers\s+(\d+)', 0)
test.file_grep(test.stats, r'Scheduling, \'act\' sense triggers\s+(\d+)', 3)
test.file_grep(test.stats, r'Procedures needing initial NBA trigger\s+(\d+)', 100)

test.execute()

test.passes()
