// DESCRIPTION: Verilator: FSM register wrapper VLT bad descriptor coverage
//
// This file ONLY is placed under the Creative Commons Public Domain.
// SPDX-FileCopyrightText: 2026 Wilson Snyder
// SPDX-License-Identifier: CC0-1.0

module t;
endmodule
