#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2026 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.top_filename = "t/t_sched_ico_change_detect_input_assigned.v"

test.compile(make_top_shell=False,
             make_main=False,
             verilator_flags2=[
                 "--exe", "--stats", "-Wno-ASSIGNIN",
                 "t/t_sched_ico_change_detect_input_assigned.cpp", "-fno-ico-change-detect"
             ])

test.execute()

test.file_grep(test.stats, r"Scheduling, 'ico' change detect triggers\s+(\d+)", 0)

test.passes()
