#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.run(cmd=[os.environ["VERILATOR_ROOT"] + "/bin/verilator_coverage", '--invalid-dash'],
         logfile=test.run_log_filename,
         fails=True,
         expect_filename=test.golden_filename,
         verilator_run=True)

test.passes()
