#!/usr/bin/env python3
# DESCRIPTION: Verilator: Verilog Test driver/expect definition
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of either the GNU Lesser General Public License Version 3
# or the Perl Artistic License Version 2.0.
# SPDX-FileCopyrightText: 2024 Wilson Snyder
# SPDX-License-Identifier: LGPL-3.0-only OR Artistic-2.0

import vltest_bootstrap

test.scenarios('vlt')

test.setenv("ASAN_OPTIONS", "handle_segv=0")
test.leak_check_disable()

test.lint(v_flags=["--debug-sigsegv"], fails='any')

test.file_grep(test.compile_log_filename,
               r'%Error: Verilator internal fault, sorry. Suggest trying --debug --gdbbt')
test.file_grep(test.compile_log_filename, r'Command Failed')

test.passes()
